##################################################################
##  
##  Aseryla project [http://aseryla.16mb.com]
##  Information Extraction task
##
##  This is the main project to get a huge number of English declarative sentences.
##  That it could be analysed to extract their characteristic for populate the memory.
##
##
##  The flow / loop is:
##      - from a webpage url (that has not been processed yet)
##           1. get their text content (discarding images, metadata, html tags and non English pages)
##           2. to keep it off the web exploration identify the links the webpage has 
##				(discarding the duplicated and the already processed or pending)
##      - given that text 
##           1. split into cleaned sentences (aseryla server: Tp order)
##           2. process those sentences, saving their extracted knowledge in the aseryla memory system (Tb order, discarding no ilc/only NER sentences)
##          
##      - save the results of the processing the page in a log/report for future porpuses as stats or loading the Aseryla system
##
##      - each x processed pages, backup the data (visited pages, 
##
##      - begin the loop for the next identified url link
##
 
 
  

from urllib2 import urlopen # http management
from bs4 import BeautifulSoup # text extraction from html code
import sys      #
import time     # 
#from stat import S_ISREG, ST_CTIME, ST_MODE  # to read files 
import hashlib  # MD5 generation
import random   # to choose the next url
import re       # regular expressions
import socket   # aseryla connections

import termios
 # to catch keyboard interruptions
from select import select
 

aserPort = 9991 # assumes the aseryla server is running whit this "non-standard" ports and autosave = 0(disabled)
nlpPort = 9992 
 
 
 
 
##  UTILS AND AUXILIARY COMMON FUNCTIONS ####################################################################################################################
 
# file management
def loadListFile(filename):
    try:
        f = open(filename, 'r')
        content = eval(f.read())
        f.close()
    except:
        content = {}
         
    return content
 
def saveListFile(content, filename):
    f = open (filename, 'w')
    f.write(str(content))
    f.close()
 
 
def appendFile(content, filename):
    f = open(filename, 'a')
    f.write(content)
    f.close()
 
 
def saveFile(filename, content):
    f = open(filename, 'wt')
    f.write(content)
    f.close()
 
 
 
 
# log formatter
def log_entry(link, tipe, sentences, notempty, totaltime):
    appendFile(link + ", " + str(tipe) + ", " + str(sentences) + ", " + str(notempty) + ", " + totaltime + "\n", "webextractor.log")
     
 
 
# convert milliseconds time to natural format in seconds
def trans_time(start):
#    return "%6.3f" % (float(int(round((time.clock() - start) * 1000))) / 1000)  windows
     return str(int(time.time() - start))


# capture keyboard
def kbhit():

    dr,dw,de = select([sys.stdin], [], [], 0)

    return dr <> []

    

    
## -------------------------------------------------------------------------------------------------------------------------------------
 
 
 
 
 
 
 
 
##  LANGUAGE HELPER ####################################################################################################################
 
 
 
class PorterStemmer:
 
    def __init__(self):
        self.b = ""  
        self.k = 0
        self.k0 = 0
        self.j = 0  
 
    def cons(self, i):
        if self.b[i] == 'a' or self.b[i] == 'e' or self.b[i] == 'i' or self.b[i] == 'o' or self.b[i] == 'u': return 0
        if self.b[i] == 'y':
            if i == self.k0: return 1
            else: return (not self.cons(i - 1))
        return 1
 
    def m(self):
        n = 0
        i = self.k0
        while 1:
            if i > self.j:
                return n
            if not self.cons(i):
                break
            i = i + 1
        i = i + 1
        while 1:
            while 1:
                if i > self.j:
                    return n
                if self.cons(i):
                    break
                i = i + 1
            i = i + 1
            n = n + 1
            while 1:
                if i > self.j:
                    return n
                if not self.cons(i):
                    break
                i = i + 1
            i = i + 1
 
    def vowelinstem(self):
        for i in range(self.k0, self.j + 1):
            if not self.cons(i): return 1
        return 0
 
    def doublec(self, j):
        if j < (self.k0 + 1): return 0
        if (self.b[j] != self.b[j-1]): return 0
        return self.cons(j)
 
    def cvc(self, i):
        if i < (self.k0 + 2) or not self.cons(i) or self.cons(i-1) or not self.cons(i-2): return 0
        ch = self.b[i]
        if ch == 'w' or ch == 'x' or ch == 'y': return 0
        return 1
 
    def ends(self, s):
        length = len(s)
        if s[length - 1] != self.b[self.k]: return 0
        if length > (self.k - self.k0 + 1): return 0
        if self.b[self.k-length+1:self.k+1] != s: return 0
        self.j = self.k - length
        return 1
 
    def setto(self, s):
        length = len(s)
        self.b = self.b[:self.j+1] + s + self.b[self.j+length+1:]
        self.k = self.j + length
 
    def r(self, s):
        if self.m() > 0: self.setto(s)
 
    def step1ab(self):
        if self.b[self.k] == 's':
            if self.ends("sses"):
                self.k = self.k - 2
            elif self.ends("ies"):
                self.setto("i")
            elif self.b[self.k - 1] != 's':
                self.k = self.k - 1
        if self.ends("eed"):
            if self.m() > 0:
                self.k = self.k - 1
        elif (self.ends("ed") or self.ends("ing")) and self.vowelinstem():
            self.k = self.j
            if self.ends("at"):   self.setto("ate")
            elif self.ends("bl"): self.setto("ble")
            elif self.ends("iz"): self.setto("ize")
            elif self.doublec(self.k):
                self.k = self.k - 1
                ch = self.b[self.k]
                if ch == 'l' or ch == 's' or ch == 'z':
                    self.k = self.k + 1
            elif (self.m() == 1 and self.cvc(self.k)):
                self.setto("e")
 
    def step1c(self):
        if (self.ends("y") and self.vowelinstem()):
            self.b = self.b[:self.k] + 'i' + self.b[self.k+1:]
 
    def step2(self):
        if self.b[self.k - 1] == 'a':
            if self.ends("ational"):   self.r("ate")
            elif self.ends("tional"):  self.r("tion")
        elif self.b[self.k - 1] == 'c':
            if self.ends("enci"):      self.r("ence")
            elif self.ends("anci"):    self.r("ance")
        elif self.b[self.k - 1] == 'e':
            if self.ends("izer"):      self.r("ize")
        elif self.b[self.k - 1] == 'l':
            if self.ends("abli"):      self.r("able")
            elif self.ends("alli"):    self.r("al")
            elif self.ends("entli"):   self.r("ent")
            elif self.ends("eli"):     self.r("e")
            elif self.ends("ousli"):   self.r("ous")
        elif self.b[self.k - 1] == 'o':
            if self.ends("ization"):   self.r("ize")
            elif self.ends("ation"):   self.r("ate")
            elif self.ends("ator"):    self.r("ate")
        elif self.b[self.k - 1] == 's':
            if self.ends("alism"):     self.r("al")
            elif self.ends("iveness"): self.r("ive")
            elif self.ends("fulness"): self.r("ful")
            elif self.ends("ousness"): self.r("ous")
        elif self.b[self.k - 1] == 't':
            if self.ends("aliti"):     self.r("al")
            elif self.ends("iviti"):   self.r("ive")
            elif self.ends("biliti"):  self.r("ble")
 
    def step3(self):
        if self.b[self.k] == 'e':
            if self.ends("icate"):     self.r("ic")
            elif self.ends("ative"):   self.r("")
            elif self.ends("alize"):   self.r("al")
        elif self.b[self.k] == 'i':
            if self.ends("iciti"):     self.r("ic")
        elif self.b[self.k] == 'l':
            if self.ends("ical"):      self.r("ic")
            elif self.ends("ful"):     self.r("")
        elif self.b[self.k] == 's':
            if self.ends("ness"):      self.r("")
 
    def step4(self):
        if self.b[self.k - 1] == 'a':
            if self.ends("al"): pass
            else: return
        elif self.b[self.k - 1] == 'c':
            if self.ends("ance"): pass
            elif self.ends("ence"): pass
            else: return
        elif self.b[self.k - 1] == 'e':
            if self.ends("er"): pass
            else: return
        elif self.b[self.k - 1] == 'i':
            if self.ends("ic"): pass
            else: return
        elif self.b[self.k - 1] == 'l':
            if self.ends("able"): pass
            elif self.ends("ible"): pass
            else: return
        elif self.b[self.k - 1] == 'n':
            if self.ends("ant"): pass
            elif self.ends("ement"): pass
            elif self.ends("ment"): pass
            elif self.ends("ent"): pass
            else: return
        elif self.b[self.k - 1] == 'o':
            if self.ends("ion") and (self.b[self.j] == 's' or self.b[self.j] == 't'): pass
            elif self.ends("ou"): pass
            else: return
        elif self.b[self.k - 1] == 's':
            if self.ends("ism"): pass
            else: return
        elif self.b[self.k - 1] == 't':
            if self.ends("ate"): pass
            elif self.ends("iti"): pass
            else: return
        elif self.b[self.k - 1] == 'u':
            if self.ends("ous"): pass
            else: return
        elif self.b[self.k - 1] == 'v':
            if self.ends("ive"): pass
            else: return
        elif self.b[self.k - 1] == 'z':
            if self.ends("ize"): pass
            else: return
        else: return
        if self.m() > 1: self.k = self.j
 
    def step5(self):
        self.j = self.k
        if self.b[self.k] == 'e':
            a = self.m()
            if a > 1 or (a == 1 and not self.cvc(self.k-1)):
                self.k = self.k - 1
        if self.b[self.k] == 'l' and self.doublec(self.k) and self.m() > 1:
            self.k = self.k -1
 
    def stem(self, p):
        self.b = p
        self.k = len(p)-1
        self.k0 = 0
        if self.k <= self.k0 + 1: return self.b 
 
        self.step1ab()
        self.step1c()
        self.step2()
        self.step3()
        self.step4()
        self.step5()
        return self.b[self.k0:self.k+1]
 
 
 
 
# check if a text is writed in English or not
# based on check if every word in the text exist into the English dictionary
# if the number of non English language words is lower then assumes is an English text
def isEnglish(text):
 
    # extract the words in the text
    # (limited to about 10000 words, assuming is enough to detect it)
    # the comparison must be in lowercase and the punctuation and numbers must be removed
    text = re.sub("[^a-z]", ' ', text[:10000].lower())
     
    ts = []
    for t in text.split():
        try:
            t = porter.stem(str(t))
            if len(t) > 1:
                ts.append(t)
        except:
            pass
 
    text_vocab = set(ts)
    size_vocab = len(text_vocab) * 1.0
    if size_vocab == 0:
        return False
    
 
    # how percetage of words are "unknown"
    unusual = text_vocab.difference(english_vocab)
    # if the percent is not higher than a threshold
    # (it can't be zero, due to proper nouns and others)
    percent = ((size_vocab - len(unusual)) / size_vocab)
 
    #print unusual, percent
    #if size_vocab < 6: return percent > 0.49 else: return percent > 0.79
    return  percent > 0.49
     
 
## ----------------------------------------------------------------------------------------------------------------------------------------------
 
 
 
 
 
 
 
 
##  COLLECTOR ####################################################################################################################
 
 
# given an url, process their content
#
# extracts the sentences (their <body> text splited by tag content)
# and the http links that contains (for keep on surfing the web)
#
# return a type of result:
#   1 Url not available or acess error
#   2 content process error
#   3 empty content (it means not sentences identified)
#   4 non English
#   5 non html (it's an image, pdf,...)
#   6 no internet connection
#   0 everything goes OK
# and the list of sentences and links identified
#
def readWebContent(url):
 
    sentences = []
    links = []

     # get the htttp response and html content
    try:
        html = urlopen("http://" + url, timeout=7).read()
    except:
          try:
                response = urlopen('http://www.google.com',timeout=1) 
                return 1, [], [] # page not found or not accesible
          except:
              try:
                  time.sleep(5) # try again after some seconds to avoid temporal line cuts
                  response = urlopen('http://www.google.com',timeout=1) 
                  return 1, [], [] # page not found or not accesible
              except:
                  return 6, [], [] # no internet connection
        
    try:
        if not re.search('<html', html, re.IGNORECASE):
            return 5, [], []
 
        # we are only interested in the content of the <body>, and in human readible format (&nbsp; --> \t can#39;t --> can't)
        soup = BeautifulSoup(html, "lxml").body
         
        # and remove tags which content is not relative to web content
        for elem in soup.findAll(['script', 'style']):
            elem.extract()
 
        body_text = soup.getText().strip()
 
        # is an empty content?
        if len(re.sub("[^-a-zA-Z]", ' ', body_text).strip()) == 0:
            return 3, [], []
 

        # extract sentences
        # in a book, every sentence is finished with a dot (or other punctuations)
        # but in the web, usually are splitted in tags which content usually are not dot ended
        # then is better to parse and analize the content tag by tab, instead of getting all the text at once
        # because the process of "sentence splitting" is not reliable without punctuations
        #
        # ej
        #    <h2>title of the web</h2>
        #    <h3>sentence one</h3>
        #
        #   - given the text at once: "title of the web\nsentence one" --> 1 sentence
        #   - given the text tag by tag  ["title of the web", "sentence one"] --> 2 sentences
        #
        texts = soup.findAll(text=True)
 
        for t in texts:
            t = t.strip()
            # do not add empty lines, and filter those ones without enough words to be a phrase or in case contains any tag
            if len(t.split(" ")) > 1 and '<' not in t: 

                if t[-1] != '.':
                    t += "." # add a sentence separator
                t += " "
                
                try:
                    sentences.append(str(t.replace("\n", "")))
                except:
                    pass # if the sentence it has non-ascii codes, it can be discarded
  
        # empty content
        if len(sentences) == 0:
            return 3, [], []

        sentences = list(set(sentences))
        
        # not analyse anything if the web page content is not in English
        if not isEnglish(''.join(sentences)):
            return 4, [], []
       
 
        # extract links
        a_tags=soup.findAll('a')
        for a in a_tags:
            try:
                l = a['href']
                if l[0:7] == "http://" and l.find(",")==-1:   # discard secure conections (https, ssl) and other protocols as mail or ftp
                    links.append(str(l))
            except:
                pass

        links = list(set(links))
 
        # correctly processed
        return 0, sentences, links
 
    except Exception as e:
        print e
        return 2, sentences, links # error processing
 


# get the domain of an url
# www.aseryla.com/ME_memory.html?params=1   --> "aseryla"
def get_domain(url):
    sufix = url.find('/')
    if sufix > -1:
        url = url[0:sufix]

    p = url.split('.')
    if len(p) < 3:
        return p[0]
    else:
        return '.'.join(p[1:-1])
        
 
## -------------------------------------------------------------------------------------------------------------------------------------------
 
 
 
 
 
 
 
 
 
 
## Aseryla server communication ###################################################################################################################
 
EndCommSymbol = "\b"
 
# manages a socket read message
def socket_receive(sock):
    total_data=[]
    data=''
    while True:
            data=sock.recv(8192)
            if EndCommSymbol in data:
                total_data.append(data[:data.find(EndCommSymbol)])
                break
            total_data.append(data)
            if len(total_data)>1:
                #check if end_of_data was split
                last_pair=total_data[-2]+total_data[-1]
                if EndCommSymbol in last_pair:
                    total_data[-2]=last_pair[:last_pair.find(EndCommSymbol)]
                    total_data.pop()
                    break
    return ''.join(total_data)
	
 
	
 
 
# send a message to the Aseryla server, and reads the response
def send_process(order, parameter, port):
     
    try:
        # connect with the server
        sock = socket.socket()
        sock.connect(("localhost", port))
 
        #send the entire message with the \b end symbol at the end (the expected final symbol in the server)
        sock.sendall(" " + order + " " + parameter + EndCommSymbol)
 
        # ok = the results list 
        # K0 = no content
        # K1 = no nlp
        # K2 = error process
        # K3 = no server
 
        # read the response
        res = socket_receive(sock)
 
        # close the communication    
        sock.close()
 
        return res[1:-1]
    except:
        return "K3"
 
 
 
 
 
# given a text, get the cleaned and parsed sentences
def split_sentences(text):
    res = send_process("Tp", text, aserPort)
  
    if res == '-': # the results are higher than 8kb
        return "K2"
 
    if res == '/':
        return "K0"
 
    return res



   
# process a bunch of sentences
def process_text(text):

## Error: Lexical parsing
## Error: Empty content
## Error: NLP kit not available
## Error: processing grammar on ("sentence")
## Error: Semantical Failure on ("sentence")
## Error: Undefined during the language processing
## number of processed sentence
    return send_process("Tb 0 ", text, aserPort)
 
 
# 
def stop_aseryla():
    send_process("@x", " ", aserPort)

def stop_nlp():
    send_process("#exit#", " ", nlpPort)

     
# checks if the Aseryla server is running properly
def check_aser_server():
    return send_process("@h", "", aserPort) != 'K3'
  

# checks if the nlp server is running properly
def check_nlp_server():
    res = send_process("@n", "", aserPort)
    
    return res != "K3" and res[0] != '0'
  


 
## -----------------------------------------------------------------------------------------------------------------------------------------
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
####################################################################################################################
##  MAIN  ##########################################################################################################
####################################################################################################################
 
 

  
 
# surf the web and process the main extracting loop
def test(url):

    try:
        tipe, sentences, links = readWebContent(url)
        print 'read: ', types[tipe]
        print sentences
     
        if tipe != 0:
            return
                     
        packs = [] # create packs of < 8kb
        cur_pack = ""
        for snt in sentences:
            cur_pack += snt
            if len(cur_pack) > 7500:
                packs.append(cur_pack)
                cur_pack = ""

        if len(cur_pack) > 0:
            packs.append(cur_pack)

        filtered = []
        for pack in packs:            
            res = split_sentences(pack)
            if len(res) == 2:
                print res
                return

            
            snts = res[:-1].split("/")
            for snt in snts:
                words = snt.split(" ")
                lenw = len(words) 
                if lenw < 2: # sentences must have more than 1 word
                    continue
         
                if lenw > 49: # truncate the sentences up to 50 words
                    snt = ""
                    for i in range(0, 49):
                        snt = snt + words[i] + " "
                                 
                if isEnglish(snt): # filter those one in the results that are not english    
                    filtered.append(snt)
     
        print
        for i in filtered:
            print i
 
    except Exception as e:
        # on error save the current page
        print "ERROR: ", str(e)
     
     
#-------------------------------------------------------------------------------------------------------------
 
     

  
 
# surf the web and process the extracted sentences 
def surf():

    if not check_aser_server():
        print "NO SERVER"
        return 

    if not check_nlp_server():
        print "NO NLP"
        return


    send_process("@w", "0", aserPort) # disable autosave mechanism
    
		 
    start = time.time()
    time.clock()  
    lToVisit = loadListFile("tovisit.txt") # urls to process
    lVisited = loadListFile("visited.txt") # web pages alredy processed, do not revisit
    print "to visit ", len(lToVisit), " visited ", len(lVisited), " time ", trans_time(start)
 
    processed = 0
    step = 0
    recover = 0
    currentPage = ""
    currentMD5 = 0

    while True:
    
         
        ## process every link or until the "escape" is pressed
        try:    
 
            start = time.time()
            step = 0
 
            if len(lToVisit) == 0:
                print "NO MORE URLS TO VISIT"
                break
 
 
            # get next page to process and mark as already processed
            currentMD5 = random.choice(lToVisit.keys())
            currentPage = lToVisit[currentMD5]
            del lToVisit[currentMD5]
 
            lVisited[currentMD5] = 0
           
 
            # access to the http and read their text content ###############################################################
            # returns 
            #   1 Url not available or acess error
            #   2 content process error
            #   3 empty content (it means not sentences identified)
            #   4 non English
            #   5 the url has no html content, e.g.: is an image or pdf document
            #   6 no internet connection
            #   0 everything goes OK --> a list of sentences and links identified
            step = 101
            tipe, sentences, links = readWebContent(currentPage)
            step = 102
            if tipe != 0:
                step = 103
                print currentPage, " --> ", types[tipe]
 
                if tipe == 6: # no Internet, no continue
                    print "NO INTERNET "
                    recover = 1
                    break
             
                log_entry(currentPage, tipe, 0, 0, trans_time(start))
                step = 104
                continue
                 
 
            # expand the map
            step = 201
            notsamedomain = []
            samedomain = []
            cur_domains = []
            for l in lToVisit.values():
                cur_domains.append(get_domain(l))
            
            for link in links:
                link = link[7:] # remove the http://
                if len(link) == 0:
                    continue
                step = 202
                
                params = link.find('?')
                if params > -1:
                    link = link[0:params] # remove the http call parameters
                if len(link) == 0:
                    continue

                if link[-1] == '/': 
                    link = link[:-1] # remove the last slash

                step = 203
                gen_md5 = hashlib.md5(link).hexdigest()
                if  gen_md5 not in lVisited and gen_md5 not in lToVisit: # avoid duplicates and revisiting
                    if get_domain(link) in cur_domains:
                        samedomain.append([gen_md5, link])
                    else:
                        notsamedomain.append([gen_md5, link])

            # mantain a list of some url to visit
            # random choice from the identified links, but given preference to those one with a different domain
            step = 204
            random.shuffle(notsamedomain) 
            random.shuffle(samedomain)

            step = 205
            while len(lToVisit) < 15 and len(notsamedomain) > 0:
                nl = notsamedomain.pop(0)
                lToVisit[nl[0]] = nl[1]
                
            step = 206
            if len(lToVisit) < 15 and len(samedomain) > 0:
                nl = samedomain.pop(0)
                lToVisit[nl[0]] = nl[1]

 

            # clean, filter the identified sentences ###############################################################
            step = 301
            packs = [] # create packs of < 8kb
            cur_pack = ""
            for snt in sentences:
                cur_pack += snt
                if len(cur_pack) > 7500:
                    packs.append(cur_pack)
                    cur_pack = ""

            if len(cur_pack) > 0:
                packs.append(cur_pack)
            step = 302
            
            filtered = []
            for pack in packs:
                step = 303
                res = split_sentences(pack)
                step = 304
                # K0 = no sentences
                # K1 (never returned)
                # K2 = error process by socket limit size of 8Kb (avoided)
                # K3 = server not listening
                if res == "K3":
                    print "ERROR cleaning: servers not listening"
                    recover = 1
                    break              

                if res != "K0":
                    step = 305
                    snts = res[:-1].split("/")
                    for snt in snts:
                        step = 306
                        words = snt.split(" ")
                        lenw = len(words) 
                        if lenw < 2: # sentences must have more than 1 word
                            continue
     
                        if lenw > 49: # truncate the sentences up to 50 words
                            snt = ""
                            for i in range(0, 49):
                                snt = snt + words[i] + " "
                        step = 307
                        if isEnglish(snt): # filter those one in the results that are not english    
                            filtered.append(snt)
                        step = 308

            if recover == 1:
                break
 
        
            step = 309
            fsnt_size = len(filtered)
	    withrels = 0
            if fsnt_size > 0:
                step = 401
                processed = processed + 1
            
                # analyse and extract the ilc to each sentence ###############################################################
                res = process_text(". ".join(filtered))
                ## Error: Lexical parsing
                ## Error: NLP kit not available
                ## Error: processing grammar on ("sentence")
                ## Error: Semantical Failure on ("sentence")
                ## Error: Undefined during the language processing
                ## number of processed sentence (if OK)

                step = 402
                if res == "K3":
                    print "SERVER NOT AVAILABLE (K3)"
                    recover = 1

                if res[0:7] == "Error: ":
                   step = 403
                   appendFile(cur_page + "\n" + res + "\n", "failed.txt")
                   if res == "Error: NLP kit not available":
                       print "NLP NOT AVAILABLE"
                       recover = 1

                if recover == 1:
                    break

		withrels = withrels + int(res)

            
				 
            # end of page processing
            step = 501
            tt = trans_time(start)
            log_entry(currentPage, "0", fsnt_size, withrels, tt)
            print processed, currentPage, fsnt_size, withrels, tt

        
            # press escape key to exit
    	    if kbhit() and ord(sys.stdin.read(1)) == 27: 
	        break


            # auto-backup
            if processed % 100 == 0:
                step = 502
                print "backup ... "
                saveListFile(lToVisit, "tovisit.txt")
                saveListFile(lVisited, "visited.txt")
                send_process("@v", "0", aserPort) # force aseryla memory saves (no tidy up)


            step = 503

            
        except Exception as e:
            # on error save the current page
            if currentMD5 == 0:
                currentPage = 'null'
                currentMD5 = 'null'
            print "ERROR: ", currentPage, str(step), str(e)
            appendFile(currentPage + " - " + currentMD5 + " [phase:" + str(step) + "]\n" + str(e) + "\n\n", "page_failed.txt")
            pass

    # save the current data
    print "exiting ..."
 
    if recover == 1 and len(currentPage) > 0:
        lToVisit[currentMD5] = currentPage # process again this page the next time
        del lVisited[currentMD5]
 
    saveListFile(lToVisit, "tovisit.txt")
    saveListFile(lVisited, "visited.txt")
    send_process("@v", "1", aserPort) # remind this action do not cleanup the memory, it's only tidy up when the system is shutdown, and if no changes no saves
 
  
 
     
 
 
 
 
 
 
 
 
##################################
##################################
##################################
 
 
 
##  INITIALIZATIONS ####################################################################################################################
 
types = ["ok", "no available", "error content", "empty content", "no english", "no html", "no Internet"] # reding webpage errors
 
porter = PorterStemmer()
english_vocab = loadListFile("english.dic")


# to catch keyboar interruptions  (this not work using the Python Idle)
fd = sys.stdin.fileno()

new_term = termios.tcgetattr(fd)

old_term = termios.tcgetattr(fd)

new_term[3] = (new_term[3] & ~termios.ICANON & ~termios.ECHO)
termios.tcsetattr(fd, termios.TCSAFLUSH, new_term) # switch to unbuffered terminal

 


 
surf()

#test("www.theguardian.com/discussion/p/4fe25")
#test("aseryla.16mb.com")




termios.tcsetattr(fd, termios.TCSAFLUSH, old_term) # switch to normal terminal



